
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import numpy as np
import scipy.ndimage as spImg
import math





def cornerDetection(image):
    
    # Sobel:
    fx = spImg.filters.sobel(image, 1)
    fy = spImg.filters.sobel(image, 0)  
    
    fxx = fx ** 2
    fyy = fy ** 2
    fxy = fx * fy
    
    # Konvolution:   
    kernel = np.ones((3, 3))
    fxxSum = spImg.convolve(fxx, kernel)
    fyySum = spImg.convolve(fyy, kernel)
    fxySum = spImg.convolve(fxy, kernel)
    
    
    # Kovarianz Matrix und "Corner Likelihood":
    k = 0.05
    h, w = image.shape    
    C = np.zeros((h, w))
    
    for y in xrange(h):
        print y
        for x in xrange(w):
            M = np.zeros((2, 2))
            M[0, 0] = fxxSum[y, x]
            M[0, 1] = fxySum[y, x]
            M[1, 0] = fxySum[y, x]
            M[1, 1] = fyySum[y, x]
            det = np.linalg.det(M)
            trace = np.trace(M)
            C[y, x] = det - k * trace ** 2
               
            
    # Non-Maxima Unterdrueckung:
    Cclean = np.zeros_like(C)
    radius = 2
    for y in xrange(radius, h - radius):
        for x in xrange(radius, w - radius):
            part = C[y - radius:y + radius, x - radius:x + radius]
            maxVal = np.max(part)
            if maxVal == C[y, x]:
                Cclean[y, x] = C[y, x]
            
            
    # Top 200 Liste erstellen:
    topCorners = []
    for y in xrange(h):
        for x in xrange(w):
            if Cclean[y, x] > 0:
                corner = (x, y, Cclean[y, x])
                topCorners.append(corner)
    
    topCorners = sorted(topCorners, key = lambda x: x[2], reverse = True)[0:min(len(topCorners), 200)]
       
    return np.array(topCorners)



def markCorners(corners, color="r"):
    
    plt.scatter(corners[:, 0], corners[:, 1], color=color)
    


def aufgabe1():
    
    input = plt.imread('fassade1.png')
    input2 = plt.imread('fassade2.png')
     
    topCorners = cornerDetection(input)
    topCorners2 = cornerDetection(input2)
       
    plt.figure(1)
    plt.subplot(221)
    plt.imshow(input, cmap=plt.cm.gray)
    plt.subplot(222)
    plt.imshow(input, cmap=plt.cm.gray)
    markCorners(topCorners)
    plt.subplot(223)
    plt.imshow(input2, cmap=plt.cm.gray)
    plt.subplot(224)
    plt.imshow(input2, cmap=plt.cm.gray)
    markCorners(topCorners2)
    plt.show()
    
    

aufgabe1()
